/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#include "DistrhoUI.hpp"
#include "ResizeHandle.hpp"

START_NAMESPACE_DISTRHO

// --------------------------------------------------------------------------------------------------------------------
enum HeavyParams {
    HIGH,
    HIGH_CRSHR,
    HIGH_FLDR,
    HIGH_GAIN,
    HIGH_LMTR,
    HIGH_MIX,
    HIGH_SMTHR,
    HIGH_SQNC,
    LOW,
    LOW_CRSHR,
    LOW_FLDR,
    LOW_GAIN,
    LOW_LMTR,
    LOW_MIX,
    LOW_SMTHR,
    LOW_SQNC,
    MID,
    MID_CRSHR,
    MID_FLDR,
    MID_FREQ,
    MID_GAIN,
    MID_LMTR,
    MID_MIX,
    MID_SMTHR,
    MID_SQNC,
};

class ImGuiPluginUI : public UI
{
    
    float fhigh = 0.0f;
    int fhigh_crshr = 512.0;
    float fhigh_fldr = 1.0f;
    float fhigh_gain = 0.0f;
    bool fhigh_lmtr = 1.0f != 0.0f;
    float fhigh_mix = 50.0f;
    float fhigh_smthr = 1.0f;
    int fhigh_sqnc = 0.0;
    float flow = 0.0f;
    int flow_crshr = 512.0;
    float flow_fldr = 1.0f;
    float flow_gain = 0.0f;
    bool flow_lmtr = 1.0f != 0.0f;
    float flow_mix = 50.0f;
    float flow_smthr = 1.0f;
    int flow_sqnc = 0.0;
    float fmid = 0.0f;
    int fmid_crshr = 512.0;
    float fmid_fldr = 1.0f;
    float fmid_freq = 1337.0f;
    float fmid_gain = 0.0f;
    bool fmid_lmtr = 1.0f != 0.0f;
    float fmid_mix = 50.0f;
    float fmid_smthr = 1.0f;
    int fmid_sqnc = 0.0;

    ResizeHandle fResizeHandle;

    // ----------------------------------------------------------------------------------------------------------------

public:
   /**
      UI class constructor.
      The UI should be initialized to a default state that matches the plugin side.
    */
    ImGuiPluginUI()
        : UI(DISTRHO_UI_DEFAULT_WIDTH, DISTRHO_UI_DEFAULT_HEIGHT),
          fResizeHandle(this)
    {
        setGeometryConstraints(DISTRHO_UI_DEFAULT_WIDTH, DISTRHO_UI_DEFAULT_HEIGHT, true);

        // hide handle if UI is resizable
        if (isResizable())
            fResizeHandle.hide();
    }

protected:
    // ----------------------------------------------------------------------------------------------------------------
    // DSP/Plugin Callbacks

   /**
      A parameter has changed on the plugin side.@n
      This is called by the host to inform the UI about parameter changes.
    */
    void parameterChanged(uint32_t index, float value) override
    {
        switch (index) {
            case HIGH:
                fhigh = value;
                break;
            case HIGH_CRSHR:
                fhigh_crshr = value;
                break;
            case HIGH_FLDR:
                fhigh_fldr = value;
                break;
            case HIGH_GAIN:
                fhigh_gain = value;
                break;
            case HIGH_LMTR:
                fhigh_lmtr = value != 0.0f;
                break;
            case HIGH_MIX:
                fhigh_mix = value;
                break;
            case HIGH_SMTHR:
                fhigh_smthr = value;
                break;
            case HIGH_SQNC:
                fhigh_sqnc = value;
                break;
            case LOW:
                flow = value;
                break;
            case LOW_CRSHR:
                flow_crshr = value;
                break;
            case LOW_FLDR:
                flow_fldr = value;
                break;
            case LOW_GAIN:
                flow_gain = value;
                break;
            case LOW_LMTR:
                flow_lmtr = value != 0.0f;
                break;
            case LOW_MIX:
                flow_mix = value;
                break;
            case LOW_SMTHR:
                flow_smthr = value;
                break;
            case LOW_SQNC:
                flow_sqnc = value;
                break;
            case MID:
                fmid = value;
                break;
            case MID_CRSHR:
                fmid_crshr = value;
                break;
            case MID_FLDR:
                fmid_fldr = value;
                break;
            case MID_FREQ:
                fmid_freq = value;
                break;
            case MID_GAIN:
                fmid_gain = value;
                break;
            case MID_LMTR:
                fmid_lmtr = value != 0.0f;
                break;
            case MID_MIX:
                fmid_mix = value;
                break;
            case MID_SMTHR:
                fmid_smthr = value;
                break;
            case MID_SQNC:
                fmid_sqnc = value;
                break;
            
            default: return;
        }
    
        repaint();
    }

    // ----------------------------------------------------------------------------------------------------------------
    // Widget Callbacks

   /**
      ImGui specific onDisplay function.
    */
    void onImGuiDisplay() override
    {
        const float width = getWidth();
        const float height = getHeight();
        const float margin = 20.0f * getScaleFactor();

        ImGui::SetNextWindowPos(ImVec2(margin, margin));
        ImGui::SetNextWindowSize(ImVec2(width - 2 * margin, height - 2 * margin));

        if (ImGui::Begin("WSTD M3NGLR", nullptr, ImGuiWindowFlags_NoResize + ImGuiWindowFlags_NoCollapse))
        {
            if (ImGui::SliderFloat("High", &fhigh, -15.0f, 15.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(HIGH, true);
                }
                setParameterValue(HIGH, fhigh);
            }
    
            if (ImGui::SliderInt("High Crshr", &fhigh_crshr, 2.0f, 512.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(HIGH_CRSHR, true);
                }
                setParameterValue(HIGH_CRSHR, fhigh_crshr);
            }
    
            if (ImGui::SliderFloat("High Fldr", &fhigh_fldr, 1.0f, 13.37f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(HIGH_FLDR, true);
                }
                setParameterValue(HIGH_FLDR, fhigh_fldr);
            }
    
            if (ImGui::SliderFloat("High Gain", &fhigh_gain, -25.0f, 0.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(HIGH_GAIN, true);
                }
                setParameterValue(HIGH_GAIN, fhigh_gain);
            }
    
            if (ImGui::Toggle("High Lmtr", &fhigh_lmtr))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(HIGH_LMTR, true);
                }
                setParameterValue(HIGH_LMTR, fhigh_lmtr);
            }
    
            if (ImGui::SliderFloat("High Mix", &fhigh_mix, 0.0f, 100.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(HIGH_MIX, true);
                }
                setParameterValue(HIGH_MIX, fhigh_mix);
            }
    
            if (ImGui::SliderFloat("High Smthr", &fhigh_smthr, 1.0f, 13.37f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(HIGH_SMTHR, true);
                }
                setParameterValue(HIGH_SMTHR, fhigh_smthr);
            }
    

            const char* high_sqnc_list[6] = {
                "C~F~S",
                "C~S~F",
                "F~C~S",
                "F~S~C",
                "S~C~F",
                "S~F~C",
            };

            if (ImGui::BeginCombo("High Sqnc", high_sqnc_list[fhigh_sqnc]))
            {
                for (int n = 0; n < 6; n++)
                {
                    bool is_selected = (fhigh_sqnc == n);
                    if (ImGui::Selectable(high_sqnc_list[n], is_selected))
                    {
                        fhigh_sqnc = n;
                        editParameter(HIGH_SQNC, true);
                        setParameterValue(HIGH_SQNC, fhigh_sqnc);
                    }
                    if (is_selected)
                        ImGui::SetItemDefaultFocus();
                }
                ImGui::EndCombo();
            }
    
            if (ImGui::SliderFloat("Low", &flow, -15.0f, 15.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(LOW, true);
                }
                setParameterValue(LOW, flow);
            }
    
            if (ImGui::SliderInt("Low Crshr", &flow_crshr, 2.0f, 512.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(LOW_CRSHR, true);
                }
                setParameterValue(LOW_CRSHR, flow_crshr);
            }
    
            if (ImGui::SliderFloat("Low Fldr", &flow_fldr, 1.0f, 13.37f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(LOW_FLDR, true);
                }
                setParameterValue(LOW_FLDR, flow_fldr);
            }
    
            if (ImGui::SliderFloat("Low Gain", &flow_gain, -25.0f, 0.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(LOW_GAIN, true);
                }
                setParameterValue(LOW_GAIN, flow_gain);
            }
    
            if (ImGui::Toggle("Low Lmtr", &flow_lmtr))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(LOW_LMTR, true);
                }
                setParameterValue(LOW_LMTR, flow_lmtr);
            }
    
            if (ImGui::SliderFloat("Low Mix", &flow_mix, 0.0f, 100.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(LOW_MIX, true);
                }
                setParameterValue(LOW_MIX, flow_mix);
            }
    
            if (ImGui::SliderFloat("Low Smthr", &flow_smthr, 1.0f, 13.37f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(LOW_SMTHR, true);
                }
                setParameterValue(LOW_SMTHR, flow_smthr);
            }
    

            const char* low_sqnc_list[6] = {
                "C~F~S",
                "C~S~F",
                "F~C~S",
                "F~S~C",
                "S~C~F",
                "S~F~C",
            };

            if (ImGui::BeginCombo("Low Sqnc", low_sqnc_list[flow_sqnc]))
            {
                for (int n = 0; n < 6; n++)
                {
                    bool is_selected = (flow_sqnc == n);
                    if (ImGui::Selectable(low_sqnc_list[n], is_selected))
                    {
                        flow_sqnc = n;
                        editParameter(LOW_SQNC, true);
                        setParameterValue(LOW_SQNC, flow_sqnc);
                    }
                    if (is_selected)
                        ImGui::SetItemDefaultFocus();
                }
                ImGui::EndCombo();
            }
    
            if (ImGui::SliderFloat("Mid", &fmid, -15.0f, 15.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MID, true);
                }
                setParameterValue(MID, fmid);
            }
    
            if (ImGui::SliderInt("Mid Crshr", &fmid_crshr, 2.0f, 512.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MID_CRSHR, true);
                }
                setParameterValue(MID_CRSHR, fmid_crshr);
            }
    
            if (ImGui::SliderFloat("Mid Fldr", &fmid_fldr, 1.0f, 13.37f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MID_FLDR, true);
                }
                setParameterValue(MID_FLDR, fmid_fldr);
            }
    
            if (ImGui::SliderFloat("Mid Freq", &fmid_freq, 313.3f, 5705.6f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MID_FREQ, true);
                }
                setParameterValue(MID_FREQ, fmid_freq);
            }
    
            if (ImGui::SliderFloat("Mid Gain", &fmid_gain, -25.0f, 0.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MID_GAIN, true);
                }
                setParameterValue(MID_GAIN, fmid_gain);
            }
    
            if (ImGui::Toggle("Mid Lmtr", &fmid_lmtr))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MID_LMTR, true);
                }
                setParameterValue(MID_LMTR, fmid_lmtr);
            }
    
            if (ImGui::SliderFloat("Mid Mix", &fmid_mix, 0.0f, 100.0f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MID_MIX, true);
                }
                setParameterValue(MID_MIX, fmid_mix);
            }
    
            if (ImGui::SliderFloat("Mid Smthr", &fmid_smthr, 1.0f, 13.37f))
            {
                if (ImGui::IsItemActivated())
                {
                    editParameter(MID_SMTHR, true);
                }
                setParameterValue(MID_SMTHR, fmid_smthr);
            }
    

            const char* mid_sqnc_list[6] = {
                "C~F~S",
                "C~S~F",
                "F~C~S",
                "F~S~C",
                "S~C~F",
                "S~F~C",
            };

            if (ImGui::BeginCombo("Mid Sqnc", mid_sqnc_list[fmid_sqnc]))
            {
                for (int n = 0; n < 6; n++)
                {
                    bool is_selected = (fmid_sqnc == n);
                    if (ImGui::Selectable(mid_sqnc_list[n], is_selected))
                    {
                        fmid_sqnc = n;
                        editParameter(MID_SQNC, true);
                        setParameterValue(MID_SQNC, fmid_sqnc);
                    }
                    if (is_selected)
                        ImGui::SetItemDefaultFocus();
                }
                ImGui::EndCombo();
            }
    
            if (ImGui::IsItemDeactivated())
            {editParameter(HIGH, false);
            editParameter(HIGH_CRSHR, false);
            editParameter(HIGH_FLDR, false);
            editParameter(HIGH_GAIN, false);
            editParameter(HIGH_LMTR, false);
            editParameter(HIGH_MIX, false);
            editParameter(HIGH_SMTHR, false);
            editParameter(HIGH_SQNC, false);
            editParameter(LOW, false);
            editParameter(LOW_CRSHR, false);
            editParameter(LOW_FLDR, false);
            editParameter(LOW_GAIN, false);
            editParameter(LOW_LMTR, false);
            editParameter(LOW_MIX, false);
            editParameter(LOW_SMTHR, false);
            editParameter(LOW_SQNC, false);
            editParameter(MID, false);
            editParameter(MID_CRSHR, false);
            editParameter(MID_FLDR, false);
            editParameter(MID_FREQ, false);
            editParameter(MID_GAIN, false);
            editParameter(MID_LMTR, false);
            editParameter(MID_MIX, false);
            editParameter(MID_SMTHR, false);
            editParameter(MID_SQNC, false);
            }
        }
        ImGui::End();
    }

    DISTRHO_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR(ImGuiPluginUI)
};

// --------------------------------------------------------------------------------------------------------------------

UI* createUI()
{
    return new ImGuiPluginUI();
}

// --------------------------------------------------------------------------------------------------------------------

END_NAMESPACE_DISTRHO